<?php

namespace SuperRhino\Utils\Auth\Middleware;

use Hyperf\Context\Context;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use SuperRhino\Utils\Auth\Auth;
use SuperRhino\Utils\Auth\AuthUtil;
use SuperRhino\Utils\Auth\Exception\AuthException;
use SuperRhino\Utils\Common\BaseMiddleware;

class AuthMiddleware extends BaseMiddleware {

    protected AuthUtil $authUtil;
    protected $scene = Auth::SCENE_APP;

    public function __construct(AuthUtil $authService) {
        $this->authUtil = $authService;
    }

    /**
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface {
        if ($this->ignore($request->getUri()->getPath())) {
            return $handler->handle($request);
        }
        $auth = $this->authUtil->handleAuth($request, $this->scene);
        if (is_null($auth)) {
            throw new AuthException('Authentication does not pass', 503);
        }
        $user_id = $auth->getUserId();
        if (empty($user_id)) {
            throw new AuthException('Authentication does not pass', 503);
        }
        Context::set(Auth::AUTH_KEY, $auth);
        return $handler->handle($request);
    }
}