<?php

namespace SuperRhino\Utils\Cache;

use Hyperf\Cache\Collector\CoroutineMemory;

class MemoryCacheHyperfAdapter implements MemoryCacheAdapterInterface {

    public function get($key, $default = null): mixed {
        return $this->getCollection()->get($key, $default);
    }

    public function getCallback($key, callable $callback, $ttl = null, ?float $beta = null) {
        if ($this->has($key)) {
            return $this->getCollection()->get($key);
        }
        $data = $callback();
        if ($data !== null) {
            $this->set($key, $data);
            return $data;
        }
        return null;
    }

    public function fetch(string $key, $default = null): array {
        if (! $this->has($key)) {
            return [false, $default];
        }

        return [true, $this->get($key)];
    }

    public function set($key, $value, $ttl = null): bool {
        return $this->getCollection()->offsetSet($key, $value);
    }

    public function delete($key): bool {
        return $this->getCollection()->offsetUnset($key);
    }

    public function clear(): bool {
        $this->getCollection()->clear();
        return true;
    }

    public function getMultiple($keys, $default = null): iterable {
        $result = [];
        foreach ($keys as $key) {
            $result[$key] = $this->get($key, $default);
        }

        return $result;
    }

    public function setMultiple($values, $ttl = null): bool {
        foreach ($values as $key => $value) {
            $this->set($key, $values, $ttl);
        }

        return true;
    }

    public function deleteMultiple($keys): bool {
        foreach ($keys as $key) {
            $this->delete($key);
        }

        return true;
    }

    public function has($key): bool {
        return $this->getCollection()->has($key);
    }

    public function clearPrefix(string $prefix): bool {
        return $this->getCollection()->clearPrefix($prefix);
    }

    public function addKey(string $collector, string $key): bool {
        $instance = $this->getCollection();
        $data = $instance->get($collector, []);
        $data[] = $key;
        $instance->put($collector, $data);

        return true;
    }

    public function keys(string $collector): array {
        return $this->getCollection()->get($collector, []);
    }

    public function delKey(string $collector, ...$key): bool {
        $instance = $this->getCollection();
        $result = [];
        $data = $instance->get($collector, []);
        foreach ($data as $item) {
            if (! in_array($item, $key)) {
                $result[] = $item;
            }
        }
        $instance->put($collector, $result);
        return true;
    }

    protected function getCollection() {
        return CoroutineMemory::instance();
    }

}